/*
   Copyright 2009,2010 Ricardo Cañuelo Navarro

   This file is part of Oolong.

   Oolong is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Oolong is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Oolong.  If not, see <http://www.gnu.org/licenses/>.
*/

package es.rediris.forja.tea.main;

import javax.swing.*;

import java.awt.*;
import java.awt.event.*;
import es.rediris.forja.tea.api.*;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.util.tracker.ServiceTracker;


/**
 * 
 * Main class. Implements BundleActivator but has no main method, so it must be called from within 
 * felix. When started it starts a GUI. It lacks a proper exit functionality
 * @author rcn, pilli
 *
 */
public class OolongGui implements BundleActivator
{
	Meter[] met;
	BundleContext ctx;
	JList pluginsList;
	ServiceTracker tracker;
	AboutBox aboutBox;
	Connect connDialog;
	
	/* Ties the context passed as argument to the ctx member variable
	 * for later use and starts the gui
	 */
	public void start(BundleContext context) throws Exception
	{
		ctx=context;
		/*
		try{
			UIManager.setLookAndFeel(
				UIManager.getSystemLookAndFeelClassName());
		}
		catch(Exception ex){
			UIManager.setLookAndFeel(
				UIManager.getCrossPlatformLookAndFeelClassName());
		}
		*/
		SwingUtilities.invokeLater(new Runnable() {
			public void run()
			{
				createGUI();
			}
		});
	}
	
	/* Nothing yet */
	public void stop(BundleContext context) throws Exception{}
	
	void createGUI()
	{
		/* widget creation */
		JFrame frame = new JFrame("Oolong");
		Image icon = Toolkit.getDefaultToolkit().getImage("images/oolong.png");
		frame.setIconImage(icon);
		JMenuBar menuBar = new JMenuBar();
		JMenu menu = new JMenu("Menu");
		JMenuItem aboutItem = new JMenuItem("Acerca de");
		JButton updateButton = new JButton("Update");
		JButton createButton = new JButton("Create");
		aboutBox = new AboutBox(frame);
		connDialog = new Connect(frame);
		menu.add(aboutItem);
		menuBar.add(menu);
		pluginsList = new JList();

		/* Set the cell renderer for the list */
		pluginsList.setCellRenderer(new DataRenderer());
		
		/* Create a service tracker to track Meter services, open it
		 * and save every service found in the plugins list
		 */
		tracker = new ServiceTracker(ctx, Meter.class.getName(), null);
		tracker.open();
		Object[] servicesArray;
		servicesArray = tracker.getServices();

		/* Create and populate the plugins list for the first time */ 
		met = new Meter[tracker.size()];
		if(servicesArray==null) {
			String[] emptyList = new String[1];
			emptyList[0] = null;
			pluginsList.setListData(emptyList);
		}
		else {
			int i=0;
			for(Object service : servicesArray) {
				met[i] = (Meter)service;
				++i;
			}
			pluginsList.setListData(met);
		}
		
		/* List scroller and plugin list attributes */
		JScrollPane listScroller = new JScrollPane(pluginsList);
		listScroller.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		listScroller.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		pluginsList.setVisibleRowCount(4);
		pluginsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		
		/* Register action listeners */
		updateButton.addActionListener(new updateListener());
		createButton.addActionListener(new createListener());
		aboutItem.addActionListener(new aboutListener());

		/* Add widgets to the main window */
		frame.getContentPane().setLayout(new FlowLayout());
		frame.getContentPane().add(updateButton);
		frame.getContentPane().add(listScroller);
		frame.getContentPane().add(createButton);
		frame.setJMenuBar(menuBar);
		frame.pack();
		frame.setResizable(false);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setVisible(true);
	}
	
	/* Inner classes for event handling */
	
	class updateListener implements ActionListener
	{
		public void actionPerformed(ActionEvent evt)
		{
			Object[] servicesArray;
			servicesArray = tracker.getServices();
			met = new Plugin[tracker.size()];
			if(servicesArray==null) {
				String[] emptyList = new String[1];
				emptyList[0] = null;
				pluginsList.setListData(emptyList);
			}
			else {
				int i=0;
				for(Object service : servicesArray) {
					met[i] = (Plugin)service;
					++i;
				}
				pluginsList.setListData(met);
			}
		}
	}
	
	class createListener implements ActionListener
	{
		public void actionPerformed(ActionEvent evt)
		{
			int index = pluginsList.getSelectedIndex();
			Plugin meter;
			if (pluginsList.getSelectedValue() != null) {
				meter = ((Plugin)pluginsList.getSelectedValue()).copy();
				connDialog.draw(meter);
			}
		}
	}
	
	class aboutListener implements ActionListener
	{
		public void actionPerformed(ActionEvent evt)
		{
			aboutBox.draw();
		}
	}

	class DataRenderer extends DefaultListCellRenderer
	{
		public Component getListCellRendererComponent(JList list,
				Object value, int index, boolean isSelected,
				 boolean cellHasFocus)
		{
			Component comp = super.getListCellRendererComponent(list, 
					value, index, isSelected, cellHasFocus);
			setPreferredSize(new Dimension(200,64));
			if(value == null) {
				((JLabel)comp).setIcon(new ImageIcon("images/vacio_logo.png"));
				((JLabel)comp).setText("No plugins");
			}
			else
				((JLabel)comp).setIcon(((Plugin)value).getIcon());
			return comp;
		}
	}
}
